/*
 * lab1.c
 */

#include "usart.h"
#include "spi.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>

#define SERVO_CENTER	2850
#define SERVO_MAX 		3200
#define SERVO_MIN		2500
const char LED_PINS[9] = {
	PB7, // front left yellow
	PB5, // front right yellow
	PB6, // rear left yellow
	PB0, // rear right yellow
	PB2, // rear left red
	PB4, // rear right red
	PB1, // front left white
	PB3, // front right white
	PA7, // front center white
};

int acc[3];
int magn[3];

void set_led(int i){
	if (i > 8 || i < 0)
		return;
	if (i == 8){
		PORTA |= (1 << LED_PINS[i]); 
	} else {
		PORTC |= (1 << LED_PINS[i]); 
	}
}
void unset_led(int i){
	if (i > 8 || i < 0)
		return;
	if (i == 8){
		PORTA &= ~(1 << LED_PINS[i]); 
	} else {
		PORTC &= ~(1 << LED_PINS[i]); 
	}
}
void init_leds(){
	// set as ouptput
	DDRC = 0xFF;
	DDRA |= (1 << PA7);
	// set all low
    PORTC = 0;
	PORTA &= ~(1 << PA7);
}

void test_leds(){
	int i;
	for (i = 0; i < 9; i++){
		set_led(i);
		_delay_ms(200);
		unset_led(i);
	}
}

void init_drive(){
	// all pins except usart pins set to output
	DDRD = 0xFF-3;
	// all pins set to low
	PORTD = 0;


	ICR1 = 12500; // 160hz
	ICR1 = 4150; // 130hz
	ICR1 = 40000;
	OCR1A = SERVO_CENTER;
	//OCR1A = 10;
    // timer1 este configurat cu modul FastPWM, top ICR1 si prescaler 8 
    TCCR1A |= (2 << WGM10) | (2 << COM1A0);
    TCCR1B |= (2 << CS10) | (3 << WGM12);

    TIMSK1 |= (1 << OCIE1A) | (1 << TOIE1);
}

void set_servo(int x){
	x -= 10;
	if (x == 0)
		OCR1A = SERVO_CENTER;
	if (x > 0){
		OCR1A = SERVO_CENTER + x * ((SERVO_MAX - SERVO_CENTER) / 9);  
	}
	if (x < 0){
		OCR1A = SERVO_CENTER + x * ((SERVO_CENTER - SERVO_MIN) / 9);  
	}
}

uint8_t read_acc(uint8_t addr){
	PORTB &= ~(1 << PB4);
	_delay_ms(2);
	addr |= 128;
	SPI_exchange(addr);
	uint8_t data = SPI_exchange(0);
	_delay_ms(2);
	PORTB |= (1 << PB4);
	_delay_ms(5);
	return data;
}

void write_acc(uint8_t addr, uint8_t data){
	PORTB &= ~(1 << PB4);
	_delay_ms(2);
	SPI_exchange(addr);
	SPI_exchange(data);
	_delay_ms(2);
	PORTB |= (1 << PB4);
	_delay_ms(5);
}

void init_acc(){
	// set as ouptput
	DDRB |= (1 << PB4);
	// set high
    PORTB |= (1 << PB4);
	SPI_init();
	write_acc(0x20, 0b01000111); // set acccel update rate to 25hz
	write_acc(0x21, 0b00000000);
	write_acc(0x24, 0b00001100); // set magn update rate to 25hz
	write_acc(0x25, 0b00100000);
	write_acc(0x26, 0b00000000);
}

void get_acc_data(){
	acc[0] = read_acc(0x28);
	acc[0] |= (read_acc(0x29) << 8);
	acc[1] = read_acc(0x2A);
	acc[1] |= (read_acc(0x2B) << 8);
	acc[2] = read_acc(0x2C);
	acc[2] |= (read_acc(0x2D) << 8);
	
	magn[0] = read_acc(0x08);
	magn[0] |= (read_acc(0x09) << 8);
	magn[1] = read_acc(0x0A);
	magn[1] |= (read_acc(0x0B) << 8);
	magn[2] = read_acc(0x0C);
	magn[2] |= (read_acc(0x0D) << 8);
}

void send_acc(){
	get_acc_data();
	char aux[16];
	sprintf(aux, "%d ", acc[0]);
	USART0_print(aux);
	sprintf(aux, "%d ", acc[1]);
	USART0_print(aux);
	sprintf(aux, "%d\n", acc[2]);
	USART0_print(aux);
	sprintf(aux, "%d ", magn[0]);
	USART0_print(aux);
	sprintf(aux, "%d ", magn[1]);
	USART0_print(aux);
	sprintf(aux, "%d\n", magn[2]);
	USART0_print(aux);
}

ISR(__vector_default)
{
	if ((UCSR0A & (1<< RXC0)) != 0 ){
		uint8_t c = USART0_receive();
		if (c < 9){
			set_led(c);
			return;
		}
		if (c < 18){
			c -= 9;
			unset_led(c);
			return;
		}
		if (c >= 21 && c <= 40){
			c -= 20;
			set_servo(c);
			return;
		}
		if (c == 61){
			PORTD &= ~(1 << PD3);
			PORTD |= (1 << PD4);
		}
		if (c == 62){
			PORTD &= ~(1 << PD4);
			PORTD |= (1 << PD3);
		}
		if (c == 63){
			PORTD &= ~(1 << PD4);
			PORTD &= ~(1 << PD3);
		}
		if (c == 65){
			send_acc();
		};
	}
}

int main()
{
	init_leds();
	init_drive();
	test_leds();
    USART0_init();
	init_acc();
	sei();
	
	while(1){
		_delay_ms(80);{
			send_acc();
		}
	}
    return 0;
}